import logging, os
import click
from .scrape import fetch_metrics_url
from .display import display


def config_log(level, location):
    logging.basicConfig(
        level=level,
        filename=os.path.expanduser(location),
        filemode="a",
        format="[%(asctime)s] {%(filename)s:%(lineno)d} %(levelname)s - %(message)s",
    )


@click.command()
@click.argument("metrics-url")
@click.option(
    "--log-level", help="CRITICAL ERROR WARNING INFO DEBUG NOTSET", default="DEBUG"
)
@click.option("--log-file", help="File location to log to", default=None)
def main(metrics_url, log_level, log_file):
    if log_file:
        config_log(logging.getLevelName(log_level), log_file)
    logger = logging.getLogger(__name__)
    logger.debug(f"Logging configured with level={log_level}")
    logger.debug(f"Now fetching metrics url={metrics_url}")
    metrics = fetch_metrics_url(metrics_url)
    display(metrics)



import re
import logging
from typing import List
import prometheus_client
from rich.console import Console
from rich.text import Text

console = Console()
logger = logging.getLogger(__name__)


def display(metrics: List["prometheus_client.metrics_core.Metric"]) -> None:
    metrics = sorted(metrics, key=lambda m: m.name)

    for metric in metrics:
        display_metric(metric)


def display_metric(metric: "prometheus_client.metrics_core.Metric") -> None:
    name = Text(metric.name)
    # bold the common prefix which indicates the metrics group

    name.stylize("yellow")

    matcher = re.match(r"(.*?)_", metric.name)
    if matcher:
        prefix = len(matcher.group(1))
        name.stylize("bold", 0, prefix)

    _type = Text(metric.type)
    _type.stylize("blue")

    unit = Text(metric.unit)
    unit.stylize("blue")

    total_samples = Text(f"{len(metric.samples)} total samples")
    total_samples.stylize("red")

    console.print(name, _type, unit, total_samples)

    if metric.documentation:
        doc = Text(metric.documentation)
        doc.stylize("green")
        console.print("  ", doc)

    labels = set()
    for sample in metric.samples:
        labels.update(sample.labels.keys())
    labels_text = Text(",".join(labels))
    if labels_text:
        labels_text.stylize("grey54")
        console.print("  ", labels_text)



import logging
import requests
from . import exceptions
import prometheus_client
from prometheus_client.parser import text_string_to_metric_families
from typing import List

logger = logging.getLogger(__name__)


def fetch_metrics_url(url) -> List["prometheus_client.metrics_core.Metric"]:
    resp = requests.get(url)

    if resp.status_code != 200:
        raise exceptions.MetricsFetchException(
            f"Can not fetch exceptions, status_code={resp.status_code},"
            f" error={resp.text}"
        )

    metrics_text = resp.text

    logger.info(f"Successfully fetch the metrics, {len(metrics_text)=}, parsing...")

    metrics = text_string_to_metric_families(metrics_text)
    # merge metrics with the same name, it seems that prometheus_client didn't
    # merge them correctly

    merge_dict = {}
    for m in metrics:
        merge_dict.setdefault(m.name, []).append(m)

    unique_metrics = list()
    for values in merge_dict.values():
        samples = []
        for m in values:
            samples.extend(m.samples)

        selected_metric = _select_metric(values)
        selected_metric.samples = samples
        unique_metrics.append(selected_metric)
    return unique_metrics


def _select_metric(values):
    for v in values:
        if v.type != "unknown":
            return v
    return values[0]



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['mepe']

package_data = \
{'': ['*']}

install_requires = \
['click>=8.1.3,<9.0.0',
 'prometheus-client>=0.15.0,<0.16.0',
 'requests>=2.28.1,<3.0.0',
 'rich>=12.6.0,<13.0.0']

entry_points = \
{'console_scripts': ['mepe = mepe.main:main']}

setup_kwargs = {
    'name': 'mepe',
    'version': '0.1.4',
    'description': '',
    'long_description': '# Prometheus Metrics Explorer\n\nCli Prometheus metrics viewer.\n\nSummaries metrics, useful when you configure a new Grafana dashboard for a new component, and want to check what metrics does it have.\n\n## Installation\n\n```shell\npip install mepe\n```\n\n## Usage\n\n```shell\nmepe http://127.0.0.1:9100/metrics\n```\n\n\n![](./docs/mepe.png)\n',
    'author': 'laixintao',
    'author_email': 'laixintaoo@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
